@@ -64,12 +64,16 @@ class ResponseBodyHandler:
         def __init__(self, *args, **kargs):
             JSONDecoder.__init__(self, *args, object_hook=_date_parser, **kargs)
 
-    def __init__(self, brotli:bool=True):
+    def __init__(self, use_brotli:bool=True):
 
-        if brotli is True:
+        if use_brotli is True:
             try:
-                import brotli
-                self.SUPPORTED_COMPRESSION.insert(0, 'br')
+                try:
+                    import brotlicffi as brotli
+                    self.SUPPORTED_COMPRESSION.insert(0, 'br')
+                except ImportError:
+                    import brotli
+                    self.SUPPORTED_COMPRESSION.insert(0, 'br')
             except ImportError:
                 pass
 
@@ -182,8 +186,18 @@ class ScrapeApiResponse:
 
     @property
     def status_code(self) -> int:
+        """
+            /!\ This is the status code of our API, not the upstream website
+        """
         return self.response.status_code
 
+    @property
+    def upstream_status_code(self) -> Optional[int]:
+        if 'status_code' in self.scrape_result:
+            return self.scrape_result['status_code']
+
+        return None
+
     @property
     def headers(self) -> CaseInsensitiveDict:
         return self.response.headers
