@@ -1,4 +1,3 @@
-from collections import Iterable
 from concurrent.futures.thread import ThreadPoolExecutor
 
 import asyncio
@@ -12,7 +11,7 @@ from io import BytesIO
 import backoff
 from requests import Session, Response
 from requests import exceptions as RequestExceptions
-from typing import TextIO, Union, List, Dict, Optional, Set
+from typing import TextIO, Union, List, Dict, Optional, Set, Iterable
 import requests
 import urllib3
 import logging as logger
@@ -25,7 +24,7 @@ except ImportError:
 from .errors import *
 from .api_response import ResponseBodyHandler
 from .scrape_config import ScrapeConfig
-from . import __version__ as version, ScrapeApiResponse
+from . import __version__ as version, ScrapeApiResponse, HttpError, UpstreamHttpError
 
 logger.getLogger(__name__)
 
@@ -75,7 +74,7 @@ class ScrapflyClient:
         self.read_timeout = read_timeout
         self.max_concurrency = max_concurrency
         self.distributed_mode = distributed_mode
-        self.body_handler = ResponseBodyHandler(brotli=brotli)
+        self.body_handler = ResponseBodyHandler(use_brotli=brotli)
         self.async_executor = ThreadPoolExecutor()
         self.http_session = None
         self.ua = 'ScrapflySDK/%s (Python %s, %s, %s)' % (
@@ -132,22 +131,27 @@ class ScrapflyClient:
     def resilient_scrape(
         self,
         scrape_config:ScrapeConfig,
-        retry_on_errors:Optional[Set[Exception]]=None,
+        retry_on_errors:Set[Exception]={ScrapflyError},
+        retry_on_status_code:Optional[List[int]]=None,
         tries: int = 5,
         delay: int = 20,
     ) -> ScrapeApiResponse:
+        assert retry_on_errors is not None, 'Retry on error is None'
+        assert isinstance(retry_on_errors, set), 'retry_on_errors is not a set()'
 
-        if not isinstance(retry_on_errors, Set) and isinstance(retry_on_errors, Iterable): # keep compat
-            retry_on_errors = set(retry_on_errors)
-
-        retryable_errors = retry_on_errors or {ScrapflyError}
-
-        if retry_on_errors is not None:
-            retryable_errors += retry_on_errors
-
-        @backoff.on_exception(backoff.expo, exception=retryable_errors, max_tries=tries, max_time=delay)
+        @backoff.on_exception(backoff.expo, exception=tuple(retry_on_errors), max_tries=tries, max_time=delay)
         def inner() -> ScrapeApiResponse:
-            return self.scrape(scrape_config=scrape_config)
+
+            try:
+                return self.scrape(scrape_config=scrape_config)
+            except (UpstreamHttpClientError, UpstreamHttpServerError) as e:
+                if retry_on_status_code is not None and e.api_response:
+                    if e.api_response.upstream_status_code in retry_on_status_code:
+                        raise e
+                    else:
+                        return e.api_response
+
+                raise e
 
         return inner()
 
@@ -207,8 +211,6 @@ class ScrapflyClient:
         request_data = self._scrape_request(scrape_config=scrape_config)
         response = self._http_handler(**request_data)
 
-        logger.info(response.headers)
-
         return self._handle_response(response=response, scrape_config=scrape_config)
 
     def _handle_response(self, response:Response, scrape_config:ScrapeConfig) -> ScrapeApiResponse:
@@ -234,18 +236,20 @@ class ScrapflyClient:
                     api_response.result['result']['duration'])
                 )
 
+                logger.debug('Log url: %s' % api_response.result['result']['log_url'])
+
             return api_response
-        except UpstreamHttpClientError as e:
+        except UpstreamHttpError as e:
             if scrape_config.method == 'HEAD':
                 logger.warning('<-- %s | %s' % (str(e), e.api_response.response.request.url))
             else:
                 logger.warning('<-- %s | %s' % (str(e), e.api_response.result['result']['url']))
             raise
-        except ApiHttpServerError as e:
-            logger.critical('<-- %s' % str(e))
+        except HttpError as e:
+            logger.critical('<-- %s - %s' % (e.response.status_code, str(e)))
             raise
         except ScrapflyError as e:
-            logger.critical('<-- - %s' % (str(e)))
+            logger.critical('<-- %s' % (str(e)))
             raise
 
     def save_screenshot(self, api_response:ScrapeApiResponse, name:str, path:Optional[str]=None):
